# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""CLI interface for Invenio 3 to load legacy dump."""

from __future__ import absolute_import, print_function

import json
import sys

import click
from celery import chain
from flask_cli import with_appcontext

from .proxies import current_migrator
from .tasks.records import import_record


@click.group()
def dumps():
    """Migration commands."""


@dumps.command()
@click.argument('source', type=click.File('r'), default=sys.stdin)
@click.option('--source-type', '-t',  type=click.Choice(['json', 'marcxml']),
              default='marcxml',
              help='Whether to use JSON or MARCXML.')
@with_appcontext
def loadrecords(source, source_type):
    """Load records migration dump."""
    click.echo('Loading dump...')
    data = json.load(source)

    click.echo('Sending tasks to queue...')
    with click.progressbar(data) as records:
        for item in records:
            if current_migrator.records_post_task:
                chain(
                    import_record.s(item, source_type=source_type),
                    current_migrator.records_post_task.s()
                )()
            else:
                import_record.delay(item, source_type=source_type)


@dumps.command()
@click.argument('source', type=click.File('r'), default=sys.stdin)
@click.option('--recid', type=int)
@click.option('--files', 'entity', flag_value='files')
@click.option('--json', 'entity', flag_value='json')
@click.option('--marcxml', 'entity', flag_value='marcxml')
@with_appcontext
def inspectrecords(source, recid, entity=None):
    """Inspect records in a migration dump."""
    click.echo('Loading dump...')
    data = json.load(source)

    # Just print record identifiers if none are selected.
    if not recid:
        click.secho('Record identifiers', fg='green')
        total = 0
        for r in (d['recid'] for d in data):
            click.echo(r)
            total += 1
        click.echo('{0} records found in dump.'.format(total))
        return

    data = list(filter(lambda d: d['recid'] == recid, data))

    if not data:
        click.secho("Record not found.", fg='yellow')
        return

    for record in data:
        if entity is None:
            click.echo(json.dumps(record, indent=2))
        if entity == 'files':
            click.secho('Files', fg='green')
            click.echo(
                json.dumps(record['files'], indent=2))

        if entity == 'json':
            click.secho('Records (JSON)', fg='green')
            for revision in record['record']:
                click.secho(
                    'Revision {0}'.format(revision['modification_datetime']),
                    fg='yellow')
                click.echo(json.dumps(revision['json'], indent=2))

        if entity == 'marcxml':
            click.secho('Records (MARCXML)', fg='green')
            for revision in record['record']:
                click.secho(
                    'Revision {0}'.format(revision['marcxml']),
                    fg='yellow')
                click.echo(revision)


@dumps.command()
@click.argument('source', type=click.File('r'), default=sys.stdin)
@click.argument('logos_dir', type=click.Path(exists=True), default=None)
@with_appcontext
def loadcommunities(source, logos_dir):
    """Load communities."""
    from invenio_migrator.tasks.communities import load_community
    click.echo('Loading dump...')
    data = json.load(source)
    with click.progressbar(data) as communities:
        for c in communities:
            load_community(c, logos_dir)


@dumps.command()
@click.argument('source', type=click.File('r'), default=sys.stdin)
@with_appcontext
def loadfeatured(source):
    """Load community featurings."""
    from invenio_migrator.tasks.communities import load_featured
    click.echo('Loading dump...')
    data = json.load(source)
    with click.progressbar(data) as featured:
        for fc in featured:
            load_featured(fc)


@dumps.command()
@click.argument('source', type=click.File('r'), default=sys.stdin)
@with_appcontext
def loadusers(source):
    """Load users."""
    from .tasks.users import load_user
    click.echo('Loading dump...')
    data = json.load(source)
    with click.progressbar(data) as users:
        for u in users:
            load_user(u)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Utilities for migrating past Invenio versions to Invenio 3.0."""

from __future__ import absolute_import, print_function

from werkzeug.utils import cached_property, import_string

from .cli import dumps
from .records import RecordDump, RecordDumpLoader


def config_imp_or_default(app, config_var_imp, default):
    """Import config var import path or use default value."""
    imp = app.config.get(config_var_imp)
    return import_string(imp) if imp else default


class _InvenioMigratorState(object):
    """State object for migrator."""

    def __init__(self, app):
        """State initialization."""
        self.app = app

    @cached_property
    def records_dump_cls(self):
        return config_imp_or_default(
            self.app, 'MIGRATOR_RECORDS_DUMP_CLS', RecordDump)

    @cached_property
    def records_dumploader_cls(self):
        return config_imp_or_default(
            self.app, 'MIGRATOR_RECORDS_DUMPLOADER_CLS', RecordDumpLoader)

    @cached_property
    def records_pid_fetchers(self):
        return [
            import_string(imp) for imp in
            self.app.config.get('MIGRATOR_RECORDS_PID_FETCHERS', [])
        ]

    @cached_property
    def records_post_task(self):
        return config_imp_or_default(
            self.app, 'MIGRATOR_RECORDS_POST_TASK', None)


class InvenioMigrator(object):
    """Invenio-Migrator extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self._state = self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app.config)
        state = _InvenioMigratorState(app)
        app.extensions['invenio-migrator'] = state
        app.cli.add_command(dumps)
        return state

    def init_config(self, config):
        """Initialize config."""
        config.setdefault('MIGRATOR_FILES_POST_TASK', None)
        config.setdefault('MIGRATOR_RECORDS_POST_TASK', None)
        config.setdefault('MIGRATOR_RECORDS_PID_FETCHERS', [])

    def __getattr__(self, name):
        """Proxy to state object."""
        return getattr(self._state, name, None)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Celery task for records migration."""

from __future__ import absolute_import, print_function

from os.path import isfile, join

from celery import shared_task
from dateutil.parser import parse as iso2dt
from invenio_communities.models import Community, FeaturedCommunity
from invenio_communities.utils import save_and_validate_logo
from invenio_db import db

from .utils import iso2dt_or_none, logo_ext_wash


@shared_task()
def load_community(data, logos_dir):
    """Load community from data dump.

    :param data: Dictionary containing community data.
    :type data: dict
    :param logos_dir: Path to a local directory with community logos.
    :type logos_dir: str
    """
    logo_ext_washed = logo_ext_wash(data['logo_ext'])
    c = Community(
        id=data['id'],
        id_user=data['id_user'],
        title=data['title'],
        description=data['description'],
        page=data['page'],
        curation_policy=data['curation_policy'],
        last_record_accepted=iso2dt_or_none(data['last_record_accepted']),
        logo_ext=logo_ext_washed,
        ranking=data['ranking'],
        fixed_points=data['fixed_points'],
        created=iso2dt(data['created']),
        updated=iso2dt(data['last_modified']),
    )
    logo_path = join(logos_dir, "{0}.{1}".format(c.id, logo_ext_washed))
    db.session.add(c)
    if isfile(logo_path):
        with open(logo_path, 'rb') as fp:
            save_and_validate_logo(fp, logo_path, c.id)
    db.session.commit()


@shared_task()
def load_featured(data):
    """Load community featuring from data dump.

    :param data: Dictionary containing community featuring data.
    :type data: dict
    """
    obj = FeaturedCommunity(id=data['id'],
                            id_community=data['id_community'],
                            start_date=iso2dt(data['start_date']))
    db.session.add(obj)
    db.session.commit()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Simple script to dump all records in an Invenio instance to JSON."""

from __future__ import absolute_import, print_function

import json

import click

from .utils import collect_things_entry_points, grouper, init_app_context


@click.group()
def cli():
    """Command for dumping all records as JSON."""
    pass


@cli.command()
@click.argument('thing')
@click.option('-q', '--query', default='')
@click.option('-f', '--from-date', default='1970-01-01 00:00:00')
@click.option('--file-prefix', default=None)
@click.option('--chunk-size', default=1000)
@click.option('-a', '--argument', multiple=True)
def dump(thing, query, from_date, file_prefix, chunk_size, argument):
    """Dump data from Invenio legacy."""
    init_app_context()

    file_prefix = file_prefix if file_prefix else '{0}_dump'.format(thing)
    kwargs = dict(arg.split('=') for arg in argument)

    try:
        thing_func = collect_things_entry_points()[thing]
    except KeyError:
        click.Abort(
            '{0} is not in the list of available things to migrate: '
            '{1}'.format(thing, collect_things_entry_points()))

    click.echo("Querying {0}...".format(thing))
    count, items = thing_func.get(query, from_date, **kwargs)

    progress_i = 0  # Progress bar counter
    click.echo("Dumping {0}...".format(thing))
    with click.progressbar(length=count) as bar:
        for i, chunk_ids in enumerate(grouper(items, chunk_size)):
            with open('{0}_{1}.json'.format(file_prefix, i), 'w') as fp:
                fp.write("[\n")
                for _id in chunk_ids:
                    try:
                        json.dump(
                            thing_func.dump(_id, from_date, **kwargs), fp)
                        fp.write(",")
                    except Exception as e:
                        click.secho("Failed dump {0} {1} ({2})".format(
                            thing, _id, e.message), fg='red')
                    progress_i += 1
                    bar.update(progress_i)

                # Strip trailing comma.
                fp.seek(fp.tell()-1)
                fp.write("\n]")


@cli.command()
@click.argument('thing')
def check(thing):
    """Check data in Invenio legacy."""
    init_app_context()

    try:
        thing_func = collect_things_entry_points()[thing]
    except KeyError:
        click.Abort(
            '{0} is not in the list of available things to migrate: '
            '{1}'.format(thing, collect_things_entry_points()))

    click.echo("Querying {0}...".format(thing))
    count, items = thing_func.get_check()

    i = 0
    click.echo("Checking {0}...".format(thing))
    with click.progressbar(length=count) as bar:
        for _id in items:
            thing_func.check(_id)
            i += 1
            bar.update(i)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Module tests."""

from __future__ import absolute_import, print_function

from os.path import join

from click.testing import CliRunner

from invenio_migrator.cli import inspectrecords, loadrecords


def test_inspectrecords(script_info, datadir):
    """Test inspect records CLI."""
    runner = CliRunner()
    filepath = join(datadir, 'records.json')

    result = runner.invoke(inspectrecords, [filepath], obj=script_info)
    assert result.exit_code == 0

    result = runner.invoke(
        inspectrecords, ['--recid', '11782', filepath], obj=script_info)
    assert result.exit_code == 0

    result = runner.invoke(
        inspectrecords, ['--recid', '11782', '--files', filepath],
        obj=script_info)
    assert result.exit_code == 0

    result = runner.invoke(
        inspectrecords, ['--recid', '11782', '--json', filepath],
        obj=script_info)
    assert result.exit_code == 0

    result = runner.invoke(
        inspectrecords, ['--recid', '11782', '--marcxml', filepath],
        obj=script_info)
    assert result.exit_code == 0

    result = runner.invoke(
        inspectrecords, ['--recid', '12345', '--files', filepath],
        obj=script_info)
    print(result.output)
    assert result.exit_code == 0


def test_loadrecords(db, dummy_location, script_info, datadir):
    """Test load records CLI."""
    runner = CliRunner()
    filepath = join(datadir, 'records.json')

    result = runner.invoke(loadrecords, [filepath], obj=script_info)
    assert result.exit_code == 0



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Community dump functions."""

from __future__ import absolute_import, print_function

from .utils import dt2iso_or_empty


def get(*args, **kwargs):
    """Get users."""
    from invenio.modules.accounts.models import User
    q = User.query
    return q.count(), q.all()


def dump(u, from_date, with_json=True, latest_only=False, **kwargs):
    """Dump the users as a list of dictionaries.

    TODO: This is only dumping user info without passwords.

    :param u: User to be dumped.
    :type u: `invenio_accounts.models.User [Invenio2.x]`
    :returns: User serialized to dictionary.
    :rtype: dict
    """
    return dict(id=u.id,
                email=u.email,
                note=u.note,
                given_names=u.given_names,
                family_name=u.family_name,
                settings=u.settings,
                nickname=u.nickname,
                last_login=dt2iso_or_empty(u.last_login))



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

""""Helper methods."""

from __future__ import absolute_import, print_function

from itertools import islice

from dateutil.tz import tzlocal, tzutc
from pkg_resources import iter_entry_points


def grouper(iterable, n):
    """Create chunks of size `n` from the iterable.

    Example:

    >>> grouper('ABCDEFG', 3)
    ['ABC', 'DEF', 'G']
    """
    it = iter(iterable)
    while True:
        chunk = tuple(islice(it, n))
        if not chunk:
            return
        yield chunk


def collect_things_entry_points():
    """Collect entry points."""
    things = dict()
    for entry_point in iter_entry_points(group='invenio_migrator.things'):
        things[entry_point.name] = entry_point.load()
    return things


def init_app_context():
    """Initialize app context for Invenio 2.x."""
    try:
        from invenio.base.factory import create_app
        app = create_app()
        app.test_request_context('/').push()
        app.preprocess_request()
    except ImportError:
        pass


def datetime_toutc(dt):
    """Convert local datetime to UTC."""
    return dt.replace(tzinfo=tzlocal()).astimezone(tzutc())


def dt2iso_or_empty(dt):
    """Turn datetime object into ISO string (empty if 'dt' is None).

    :param dt: Datetime object
    :type dt: datetime.datetime
    :returns: ISO-formatted date or empty string
    :rtype: str
    """
    return '' if (dt is None) else datetime_toutc(dt).isoformat()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Proxies for Invenio-Migrator."""

from __future__ import absolute_import, print_function

from os.path import splitext

import arrow
from dojson.contrib.marc21 import marc21
from dojson.contrib.marc21.utils import create_record
from invenio_db import db
from invenio_files_rest.models import Bucket, BucketTag, FileInstance, \
    ObjectVersion
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_pidstore.models import PersistentIdentifier, PIDStatus, \
    RecordIdentifier
from invenio_pidstore.resolver import Resolver
from invenio_records.api import Record
from invenio_records_files.models import RecordsBuckets
from werkzeug.utils import cached_property


class RecordDumpLoader(object):
    """Migrate a record."""

    @classmethod
    def create(cls, dump):
        """Create record based on dump."""
        dump.prepare_revisions()
        dump.prepare_pids()
        dump.prepare_files()

        # Create or update?
        existing_files = []
        if dump.record:
            existing_files = dump.record.get('files', [])
            record = cls.update_record(dump)
            pids = dump.missing_pids
        else:
            record = cls.create_record(dump)
            pids = dump.pids

        if pids:
            cls.create_pids(record.id, pids)

        if dump.files:
            cls.create_files(record, dump.files, existing_files)

        # Update files.
        if dump.is_deleted(record):
            cls.delete_record(record)

        return record

    @classmethod
    def create_record(cls, dump):
        """Create a new record from dump."""
        # Reserve record identifier, create record and recid pid in one
        # operation.
        timestamp, data = dump.pop_first_revision()
        record = Record.create(data)
        record.model.created = timestamp.replace(tzinfo=None)
        RecordIdentifier.insert(dump.recid)
        PersistentIdentifier.create(
            pid_type='recid',
            pid_value=str(dump.recid),
            object_type='rec',
            object_uuid=str(record.id),
            status=PIDStatus.REGISTERED
        )
        db.session.commit()
        return cls.update_record(dump, record=record)

    @staticmethod
    def update_record(dump, record=None):
        """Update an existing record."""
        record = record or dump.record
        for timestamp, revision in dump.revisions:
            record.model.json = revision
            record.model.created = timestamp.replace(tzinfo=None)
            db.session.commit()
        return Record(record.model.json, model=record.model)

    @staticmethod
    def create_pids(record_uuid, pids):
        """Create persistent identifiers."""
        for p in pids:
            PersistentIdentifier.create(
                pid_type=p.pid_type,
                pid_value=p.pid_value,
                pid_provider=p.provider.pid_provider if p.provider else None,
                object_type='rec',
                object_uuid=record_uuid,
                status=PIDStatus.REGISTERED,
            )
        db.session.commit()

    @classmethod
    def delete_record(cls, record):
        """Delete a record and it's persistent identifiers."""
        record.delete()
        PersistentIdentifier.query.filter_by(
            object_type='rec', object_uuid=record.id,
        ).update({PersistentIdentifier.status: PIDStatus.DELETED})
        cls.delete_buckets(record)
        db.session.commit()

    @classmethod
    def create_files(cls, record, files, existing_files):
        """Create files.

        This method is currently limited to a single bucket per record.
        """
        default_bucket = None
        # Look for bucket id in existing files.
        for f in existing_files:
            if 'bucket' in f:
                default_bucket = f['bucket']
                break

        # Create a bucket in default location if none is found.
        if default_bucket is None:
            b = Bucket.create()
            BucketTag.create(b, 'record', str(record.id))
            default_bucket = str(b.id)
            db.session.commit()
        else:
            b = Bucket.get(default_bucket)

        record['files'] = []
        for key, meta in files.items():
            obj = cls.create_file(b, key, meta)
            ext = splitext(obj.key)[1].lower()
            if ext.startswith('.'):
                ext = ext[1:]
            record['files'].append(dict(
                bucket=str(obj.bucket.id),
                filename=obj.key,
                version_id=str(obj.version_id),
                size=obj.file.size,
                checksum=obj.file.checksum,
                type=ext,
            ))
        db.session.add(
            RecordsBuckets(record_id=record.id, bucket_id=b.id)
        )
        record.commit()
        db.session.commit()

        return [b]

    @classmethod
    def create_file(self, bucket, key, file_versions):
        """Create a single file with all versions."""
        objs = []
        for file_ver in file_versions:
            f = FileInstance.create().set_uri(
                file_ver['full_path'],
                file_ver['size'],
                'md5:{0}'.format(file_ver['checksum']),
            )
            obj = ObjectVersion.create(bucket, key).set_file(f)
            obj.created = arrow.get(
                file_ver['creation_date']).datetime.replace(tzinfo=None)
            objs.append(obj)

        # Set head version
        db.session.commit()
        return objs[-1]

    @staticmethod
    def delete_buckets(record):
        """Delete the bucket."""
        files = record.get('files', [])
        buckets = set()
        for f in files:
            buckets.add(f.get('bucket'))
        for b_id in buckets:
            b = Bucket.get(b_id)
            b.deleted = True


class RecordDump(object):
    """Record dump wrapper.

    Wrapper around a record dump, with tools for loading the dump. Extend this
    class to provide custom behavior for loading of record dumps.

    Known limitations:

      - Only persistent identifiers present in the last revision of the record
        will be registered.
    """

    def __init__(self, data, source_type='marcxml', latest_only=False,
                 pid_fetchers=None, dojson_model=marc21):
        """Initialize class."""
        self.resolver = Resolver(
            pid_type='recid', object_type='rec', getter=Record.get_record)
        self.data = data
        self.source_type = source_type
        self.latest_only = latest_only
        self.dojson_model = dojson_model
        self.revisions = None
        self.pid_fetchers = pid_fetchers or []

    @cached_property
    def record(self):
        """Get the first revision."""
        try:
            return self.resolver.resolve(self.data['recid'])[1]
        except PIDDoesNotExistError:
            return None

    @cached_property
    def missing_pids(self):
        """Filter persistent identifiers."""
        missing = []
        for p in self.pids:
            try:
                PersistentIdentifier.get(p.pid_type, p.pid_value)
            except PIDDoesNotExistError:
                missing.append(p)
        return missing

    @cached_property
    def recid(self):
        """Get recid."""
        return self.data['recid']

    def _prepare_revision(self, data):
        dt = arrow.get(data['modification_datetime']).datetime

        if self.source_type == 'marcxml':
            val = self.dojson_model.do(create_record(data['marcxml']))
        else:
            val = data['json']

        return (dt, val)

    def prepare_revisions(self):
        """Prepare data."""
        # Prepare revisions
        self.revisions = []

        it = [self.data['record'][0]] if self.latest_only \
            else self.data['record']

        for i in it:
            self.revisions.append(self._prepare_revision(i))

    def prepare_files(self):
        """Get files from data dump."""
        # Prepare files
        files = {}
        for f in self.data['files']:
            k = f['full_name']
            if k not in files:
                files[k] = []
            files[k].append(f)

        # Sort versions
        for k in files.keys():
            files[k].sort(key=lambda x: x['version'])

        self.files = files

    def prepare_pids(self):
        """Prepare persistent identifiers."""
        self.pids = []
        for fetcher in self.pid_fetchers:
            val = fetcher(None, self.revisions[-1][1])
            if val:
                self.pids.append(val)

    def pop_first_revision(self):
        """Get the first revision."""
        return self.revisions.pop(0)

    def is_deleted(self, record=None):
        """Check if record is deleted."""
        record = record or self.revisions[-1][1]
        return any(
            col == 'deleted'
            for col in record.get('collections', [])
        )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Module tests."""

from __future__ import absolute_import, print_function

import hashlib
from os.path import join

from invenio_accounts.testutils import create_test_user
from invenio_communities.models import Community, FeaturedCommunity
from invenio_communities.utils import initialize_communities_bucket
from invenio_files_rest.models import Location, ObjectVersion

from invenio_migrator.tasks.communities import load_community, load_featured


def test_communities_load(db, communities_dump, featured_dump, logos_dir):
    """Load the communities JSON dump."""
    # Create a single user (with id=1) to meet the community's FK constraint.
    create_test_user('test@invenio.org', password='123456')

    # Initialize communities bucket (requires a default Location)
    loc = Location(name='local', uri='file:///tmp', default=True)
    db.session.add(loc)
    db.session.commit()
    initialize_communities_bucket()

    # Load the communities
    for data in communities_dump:
        load_community.delay(data, logos_dir)

    # Check if community was loaded correctly
    assert Community.query.count() == 1
    c = Community.query.first()
    assert c.title == 'Invenio Community'
    assert c.id == 'invenio'

    # Check object metadata
    assert ObjectVersion.query.count() == 1  # Check if the logo was created
    assert ObjectVersion.query.first().key == 'invenio/logo.jpg'

    # Open the original logo from test data and check if checksums match
    with open(join(logos_dir, 'invenio.jpg'), 'rb') as fp:
        logo_checksum = hashlib.md5(fp.read()).hexdigest()
        assert ObjectVersion.query.first().file.checksum == \
            'md5:{0}'.format(logo_checksum)

    # Load the community featurings
    for data in featured_dump:
        load_featured.delay(data)

    # Make sure they are loaded correctly
    assert FeaturedCommunity.query.count() == 1
    fc = FeaturedCommunity.query.first()
    assert fc.community.id == c.id



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Celery task for records migration."""

from __future__ import absolute_import, print_function

import uuid
from datetime import datetime

import arrow
from celery import shared_task
from flask import current_app
from invenio_accounts.models import User
from invenio_db import db
from invenio_userprofiles.api import UserProfile


@shared_task()
def load_user(data):
    """Load user from data dump.

    :param data: Dictionary containing user data.
    :type data: dict
    """
    email = data['email']
    if User.query.filter_by(email=data['email']).count() > 0:
        email = 'DUPLICATE_{}'.format(data['email'])

    last_login = None
    if data['last_login']:
        last_login = arrow.get(data['last_login']).datetime

    confirmed_at = None
    if data['note'] == '1':
        confirmed_at = datetime.utcnow()

    with db.session.begin_nested():
        obj = User(
            id=data['id'],
            password=None,
            email=email,
            confirmed_at=confirmed_at,
            last_login_at=last_login,
            active=(data['note'] != '0'),
        )
        db.session.add(obj)

    nickname = data['nickname'].strip()
    if nickname:
        full_name = (data.get('given_names', '') + u' ' +
                     data.get('family_name', '')).strip()

        if UserProfile.query.filter(
                UserProfile._username == nickname.lower()).count() > 0:
            nickname = 'DUPLICATE_{0}_{1}'.format(str(uuid.uuid4()), nickname)

        p = UserProfile(user=obj)
        p.full_name = full_name or ''
        try:
            p.username = nickname
        except ValueError:
            current_app.logger.warn(
                u'Invalid username {0} for user_id {1}'.format(
                    nickname, data['id']))
            p._username = nickname.lower()
            p._displayname = nickname

        db.session.add(p)

    db.session.commit()
